/* Copyright (C) 2009, Arne König
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "audio.h"

// note: the next line assumes that the background music is the first sample
mm_ds_sample bgSample = {2995, 202533, 0, 1, 1<<9, (void*)&soundbank_bin};
// sample rate handle volume panning
mm_sound_effect bgSound = { (mm_word)&bgSample, 1<<10, 0, 200, 127};
mm_sound_effect toSound = { SFX_TIMEOUT, 1<<10, 0, 120, 127};
mm_sound_effect squareSound = { SFX_SQUARE, 1<<10, 0, 220, 127};
mm_sound_effect endSound = { SFX_END, 1<<10, 0, 220, 127};

Audio::Audio() {
	this->bgMusic = -1;
	mmInitDefaultMem( (mm_addr)soundbank_bin );
	mmLoadEffect(SFX_BGMUSIC);
	mmLoadEffect(SFX_SQUARE);
	mmLoadEffect(SFX_TIMEOUT);
	mmLoadEffect(SFX_END);
	this->update(ALL_OPTION);
}

void 
Audio::update(TSoundOption mode) {
	this->mode = mode;
	if (mode & MUSIC_OPTION)
		startBgMusic();
	else
		stopBgMusic();
}

void
Audio::startBgMusic() {
	if (bgMusic == -1)
		bgMusic = mmEffectEx(&bgSound);
}

void
Audio::stopBgMusic() {
	if (bgMusic != -1) {
		mmEffectCancel(bgMusic);
		bgMusic = -1;
	}
}

void
Audio::playSquareSound() {
	if (mode & EFFECTS_OPTION)
		mmEffectEx(&squareSound);
}

void
Audio::playTimeoutSound() {
	if (mode & EFFECTS_OPTION)
		mmEffectEx(&toSound);
}

void
Audio::playEndSound() {
	if (mode & EFFECTS_OPTION)
		mmEffectEx(&endSound);
}